/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */


package chat.presentation;

import com.lutris.xml.xmlc.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import chat.business.Discussion;
import chat.ChatApplication;
import org.w3c.dom.*;
import org.w3c.dom.html.*;


/**
 * The EntryForm Presnetation object preduces the HTML code of one chat frame. 
 *
 * There is no direct communication between this frame and the frame above it.
 * The user sees his message appear through the same channels as all the other
 * users. This page just submits a form like any other page.
 *
 * Because we want the entry form to re-appear after the data is submitted, 
 * so the usr can send in another message, we set the form to submit the data to
 * this page. The first request we get will just be a normal GET with no 
 * arguments.The second one will be both a command to add a message and a request 
 * for the entry form. So we look for optional parameters, if they are there we
 * do the action, then either way we emit the form.
 *
 * The action we (optionally) do is to process a message. Normally this means
 * adding it to the Discussion business object (which takes care of all the
 * notifying; the call always returns with no blocking). However, if the
 * message is the reset string (optional part of the config file, get it from
 * the application object) we tell the Discucion business object to clear out
 * it's list of messages (again, it takes care of all the notifying).
 */
public class EntryForm implements HttpPresentation {

   /**
    *  If it is not the first request from a user (or a refresh), then 
    *  we are being called by the form we emitted last time. In this case
    *  we should process the message being sent in. If it's the clear
    *  command (optional), then clear the message list. Most of the time
    *  we just add the message to the current discussion. Note how the
    *  HTML/HTTP and the logic of keeping messages is kept separate.
    */
    public void run(HttpPresentationComms comms) throws Exception {
        EntryFormHTML page = new EntryFormHTML();
	ChatApplication myApp = (ChatApplication) comms.application;

	// Get user name, set it up, and then send it back to the user
	String name = comms.request.getParameter("userName");
	HTMLInputElement theName = (HTMLInputElement) page.getElementById("userName");
	theName.setValue(name);

	// Send message to all chat boards
	String text = comms.request.getParameter("text");
	if ((name != null) || (text != null)) {
	    String clearCmd = myApp.getClearCommand();
	    if ((clearCmd.length() != 0) && text.equals(clearCmd)) {
		Discussion.clear();
	    }
	    else {
		Discussion.addMessage(name, text);
	    }
	}
        comms.response.writeHTML(page.toDocument());
    }
}
