/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package demoApp.business.user;

import com.lutris.appserver.server.user.*;
import demoApp.data.user.*;
import java.io.Serializable;

/**
 * The application specific user business object.
 *
 * @author Shawn McMurdo
 * @version $Revision: 1.14 $
 */
public class DemoAppUser implements User, Serializable {
    private DemoAppUserDO userDO;
    
    /**
     * Constructor
     * 
     * @param userDO A user data object representing the user's identity.
     */ 
    public DemoAppUser(DemoAppUserDO userDO) {
	this.userDO = userDO;
    }

    // BaseUser implementation methods

    /**
     * Get the user name of this user account.
     * @return The user name of this user account.
     */
    public String getName() {
	return userDO.getName();
    }

    /**
     * Process an authentication request.  Verify
     * that the user's account and password are valid.
     *
     * @param password The password entered by the user.
     * @return An int representing success (AUTH_OK), or a failure
     *   reason (AUTH_FAILED, IO_ERROR).
     */
    public int authenticate(String password) {
	if (password.compareTo(userDO.getPassword()) == 0) {
	    return DemoAppUserManager.AUTH_OK;
	} else {
	    return DemoAppUserManager.AUTH_FAILED;
	}
    }
 
    /**
     * Changes the user's password. The old password must be supplied and it
     * must match the password stored in the user database.  The new password
     * must be passed twice, and both strings must match to verify
     * correctness.  An initiating user maybe specified if a user other than
     * the one represented is this object is making the modification.
     * 
     * @param old The old password.
     * @param new1 The new password.
     * @param new2 Second copy of new password for comparison.
     * @param initiatingUser If not null, it is the user that is
     *  initiating the change.
     * 
     * @exception LoginException Thrown if the old password fails
     *   authentication.
     */
    public void changePassword(String old, String new1,
                               String new2, User initiatingUser) throws LoginException {
	// Use sessionKey to determine if we are allowed to
	// change the password.
	if (authenticate(old) == DemoAppUserManager.AUTH_OK) {
	    if (new1.compareTo(new2) == 0) {
		userDO.setPassword(new1);
	    } else {
		// FIX: throw exception
	    }
	} else {
		// FIX: throw exception
	}
    }

    // DemoAppUser specific methods

    public int getUserID() {
	return userDO.getUserID();
    }


    /**
     * Compare this object to another user object.  They are
     * equal if the user name is the same.
     */
    public boolean equals(Object obj) {
        return (obj instanceof DemoAppUser)
            && (((DemoAppUser)obj).getName().equals(getName()));
    }
}

