/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package demoApp.presentation.login;

import java.io.*;
import java.net.*;
import com.lutris.http.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.user.*;
import com.lutris.util.*;
import demoApp.DemoApp;
import demoApp.business.user.*;

/**
 *   Presentation Object that processes the login request and redirects
 *      to the appropriate page.
 *
 * @author Shawn McMurdo
 * @version $Revision: 1.24 $
 */
public class LoginProcessor implements HttpPresentation
{
    static final String badpassword = "Incorrect username or password.";
    static final String unknownhost = "Login failed: Unknown host name.";
    static final String noremotehost = "Login failed: No remote host header.";
    static final String loginfailed = "Login failed for an unknown reason.";
    static final String toomanylogins = "Login failed: Exceeded maximum number of sessions.";
    static final String permdenied = "Login failed: Permission denied.";
    static final String disabled = "Login failed: Account is disabled.";

    static final String homePage = "examples/HomePage.po";
    static final String loginPage = "login/Login.po";
    static final String errorMsgArg = "errorMsg";
    static final String debugArg = "debug";
    static final String helpArg = "help";
   


    public void run(HttpPresentationComms comms)
        throws Exception
    {
        DemoApp application = (DemoApp)comms.application;
	String cookieval = "";
	String nousername =
	    "You must specify a username and password to login.";

        String username = comms.request.getParameter("username");
        String password = comms.request.getParameter("password");
        String debug = comms.request.getParameter("debug");
        String help = comms.request.getParameter("help");

       String pp = comms.request.getPresentationPath();
       if (!pp.endsWith("/"))
           pp += "/";

        // Session has already been setup by the Application, just log it in.
	try {
	    application.getUserManager().login(username, password,
                                               comms.session);

	    // Must set content-type in java presentation objects
	    // for web servers that default to text/plain.
	    comms.response.setContentType("text/html");

            HttpPresentationOutputStream out = comms.response.getOutputStream();
            out.print("<HTML><HEAD>");
            out.print("<META HTTP-EQUIV=\"Refresh\" " +
                                 "CONTENT=\"1; URL=");
            out.print(pp + homePage + "\">");
            out.print("</HEAD></HTML>");
	}
	catch (LoginException le) {
            ClientPageRedirectException redir =
                new ClientPageRedirectException(pp + loginPage);
	    switch (le.reason) {
	    case DemoAppUserManager.UNKNOWN_ERROR:
	    case DemoAppUserManager.IO_ERROR:
		redir.addArgument(errorMsgArg, badpassword);
		redir.addArgument(debugArg, debug);
		redir.addArgument(helpArg, help);
		break;
	    case DemoAppUserManager.AUTH_FAILED:
		redir.addArgument(errorMsgArg, badpassword);
		redir.addArgument(debugArg, debug);
		redir.addArgument(helpArg, help);
		break;
	    case DemoAppUserManager.MULTIPLE_LOGIN:
		redir.addArgument(errorMsgArg, toomanylogins);
		redir.addArgument(debugArg, debug);
		redir.addArgument(helpArg, help);
		break;
	    case DemoAppUserManager.PERMISSION_DENIED:
		redir.addArgument(errorMsgArg, permdenied);
		redir.addArgument(debugArg, debug);
		redir.addArgument(helpArg, help);
		break;
	    case DemoAppUserManager.ACCOUNT_DISABLED:
		redir.addArgument(errorMsgArg, disabled);
		redir.addArgument(debugArg, debug);
		redir.addArgument(helpArg, help);
		break;
	    default:
		redir.addArgument(errorMsgArg, loginfailed);
		redir.addArgument(debugArg, debug);
		redir.addArgument(helpArg, help);
		break;
	    }
	    throw redir;
	}
    }
}

