/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.data.user;

import java.util.Vector;
import java.util.Enumeration;
import netscape.ldap.*;
import com.lutris.logging.Logger;
import com.lutris.logging.LogChannel;
import com.lutris.appserver.server.LBS;
import golfShop.data.user.UserDO;
import golfShop.data.user.UserStore;
import golfShop.GolfShop;
import com.lutris.util.ConfigException;



/**
 * This is one flavor of a UserStore. The storage medium in this case
 * is an LDAP server. 
 *
 * @author Andrew John
 * @version $Revision: 1.8 $
 */
public class LdapUserStore extends UserStore {

    private static String serverName = null;
    private static int serverPort = 0;
    private static String adminUsername = null;
    private static String adminPW = null;
    private static String serverRoot = null;

    /**
     *   Initialize the server settings. Which server to use? What port?
     *   What's the password? All these are taken from the config file.
     */
    protected void initializeUserStore() {
        try {
            serverName = GolfShop.config.getString(
                                           "GolfShop.UserStore.LdapServer");
        } catch (ConfigException e) {}
        try {
            String portS = GolfShop.config.getString(
                                             "GolfShop.UserStore.LdapPort");
            Integer portI = new Integer(portS);
            serverPort = portI.intValue();
        } catch (ConfigException e) {}
        try {
            serverRoot = GolfShop.config.getString(
                                       "GolfShop.UserStore.LdapServerRoot");
        } catch (ConfigException e) {}
        try {
            adminUsername = GolfShop.config.getString(
                                        "GolfShop.UserStore.LdapAdminName");
        } catch (ConfigException e) {}
        try {
            adminPW = GolfShop.config.getString(
                                          "GolfShop.UserStore.LdapAdminPW");
        } catch (ConfigException e) {}
        LogChannel chan = LBS.getLogChannel();
        if (chan != null)
            chan.write(Logger.INFO, "LDAP configuration:" +
                                    "\n    Server: " + serverName +
                                    "\n    Port: " + serverPort +
                                    "\n    Root: " + serverRoot +
                                    "\n    DB Admin User: " + adminUsername +
                                    "\n    DB Admin PW: " + adminPW);
    }


    protected boolean usernameInUserStore(String username) {
        LDAPConnection ldap = openLdapConnection();
        if (ldap == null) 
            return false;
        LDAPSearchConstraints cons = ldap.getSearchConstraints();
        boolean found = false;
        cons.setBatchSize( 1 );
        LDAPSearchResults res = null;
        try {
            res = ldap.search(serverRoot, LDAPv2.SCOPE_ONE,
                                "uid=" + username, null, true, cons);
        } catch (LDAPException e) {
            LogChannel chan = LBS.getLogChannel();
            if (chan != null)
                chan.write(Logger.ERROR, "LDAP search error: " +
                           lookupLdapError(e), e);
            return false;
        }
        // If there are any results at all, that answers the question.
        if (res.hasMoreElements()) {
            try {
                ldap.abandon(res);
            } catch (LDAPException e) {}
            found = true;
        }
        closeLdapConnection(ldap);
        return found;
    }

    protected UserDO lookupUserFromUserStore(String username) {
        String user = "";
        String password = "";
        String address1 = "";
        String address2 = "";
        String city = "";
        String state = "";
        String zip = "";
        String creditCard = "";
        String email = "";
        LDAPConnection ldap = openLdapConnection();
        if (ldap == null) 
            return null;
        LDAPSearchConstraints cons = ldap.getSearchConstraints();
        cons.setBatchSize(1);
        LDAPSearchResults res = null;
        try {
            res = ldap.search(serverRoot, LDAPv2.SCOPE_ONE, 
                                "uid=" + username, null, false, cons);
        } catch (LDAPException e) {
            LogChannel chan = LBS.getLogChannel();
            if (chan != null)
                chan.write(Logger.ERROR, "LDAP search error: " +
                           lookupLdapError(e), e);
            return null;
        }
        // If there are any results, just use the first one.
        if (!res.hasMoreElements()) {
            closeLdapConnection(ldap);
            return null;
        }
        LDAPEntry entry = null;
        try {
            entry = res.next();
        } catch (LDAPReferralException e) {
            closeLdapConnection(ldap);
            return null;
        }
        LDAPAttributeSet entryAttrs = entry.getAttributeSet();
        Enumeration attrsInSet = entryAttrs.getAttributes();
        while (attrsInSet.hasMoreElements()) {
            LDAPAttribute nextAttr = (LDAPAttribute) attrsInSet.nextElement();
            String attrName = nextAttr.getName();
            Enumeration valsInAttr = nextAttr.getStringValues();
            // Only care about first value.
            if (!valsInAttr.hasMoreElements()) 
                continue;
            String attrValue = (String) valsInAttr.nextElement();
            if (attrValue == null)
                attrValue = "";
            if (attrName.equalsIgnoreCase("username")) {
                user = attrValue;
            } else if (attrName.equalsIgnoreCase("password")) {
                password = attrValue;
            } else if (attrName.equalsIgnoreCase("address1")) {
                address1 = attrValue;
            } else if (attrName.equalsIgnoreCase("address2")) {
                address2 = attrValue;
            } else if (attrName.equalsIgnoreCase("city")) {
                city = attrValue;
            } else if (attrName.equalsIgnoreCase("state")) {
                state = attrValue;
            } else if (attrName.equalsIgnoreCase("zip")) {
                zip = attrValue;
            } else if (attrName.equalsIgnoreCase("creditCard")) {
                creditCard = attrValue;
            } else if (attrName.equalsIgnoreCase("email")) {
                email = attrValue;
            }
        }
        // Done.
        if (res.hasMoreElements())
            try {
                ldap.abandon(res);
            } catch (LDAPException e) {}
        closeLdapConnection(ldap);
        // See if we got enough to make a user.
        if (user.equals("") || password.equals("")) {
            LogChannel chan = LBS.getLogChannel();
            if (chan != null)
                chan.write(Logger.WARNING, 
                           "LDAP error: Bogus user in database. " +
                           "\n    username = " + user +
                           "\n    pw = " + password);
            return null;
        }
        return new UserDO(user, password, address1, address2,
                                    city, state, zip, creditCard, email);
    }
    
    protected void addUserToUserStore(UserDO user) {
        LDAPConnection ldap = openLdapConnection();
        if (ldap == null) 
            return;
        String newDN = "uid=" + user.getName() + ", " + serverRoot;
        LDAPAttributeSet attrs = new LDAPAttributeSet();
        LDAPAttribute objClass = new LDAPAttribute("objectclass");
        objClass.addValue("top");
        objClass.addValue("golfShopUser");
        attrs.add(objClass);
        LDAPAttribute userN = new LDAPAttribute("username", 
                                                    user.getName());
        attrs.add(userN);
        LDAPAttribute password = new LDAPAttribute("password", 
                                                    user.getPassword());
        attrs.add(password);
        LDAPAttribute email = new LDAPAttribute("email", user.getEmail());
        attrs.add(email);
        LDAPAttribute a1 = new LDAPAttribute("address1", user.getAddress1());
        attrs.add(a1);
        LDAPAttribute a2 = new LDAPAttribute("address2", user.getAddress2());
        attrs.add(a2);
        LDAPAttribute city = new LDAPAttribute("city", user.getCity());
        attrs.add(city);
        LDAPAttribute state = new LDAPAttribute("state", user.getState());
        attrs.add(state);
        LDAPAttribute zip = new LDAPAttribute("zip", user.getZip());
        attrs.add(zip);
        LDAPAttribute cc = new LDAPAttribute("creditCard", 
                                                    user.getCreditCard());
        attrs.add(cc);
        LDAPEntry newEntry = new LDAPEntry(newDN, attrs);
        try {
            ldap.add(newEntry);
        } catch (LDAPException e) {
            LogChannel chan = LBS.getLogChannel();
            if (chan != null)
                chan.write(Logger.ERROR, "LDAP error adding new user: " + 
                           lookupLdapError(e) +
                           "\n    New DN = " + newDN +
                           "\n    New entry = " + attrs, e);

        }
        closeLdapConnection(ldap);
    }

    protected void updateUserInUserStore(UserDO user) {
        LDAPConnection ldap = openLdapConnection();
        if (ldap == null) 
            return;
        String userDN = "uid=" + user.getName() + ", " + serverRoot;
        LDAPModificationSet mods = new LDAPModificationSet();
        LDAPAttribute attrEmail = new LDAPAttribute("email", user.getEmail());
        mods.add(LDAPModification.REPLACE, attrEmail);
        LDAPAttribute attrAd1 = new LDAPAttribute("address1", 
                                                    user.getAddress1());
        mods.add(LDAPModification.REPLACE, attrAd1);
        LDAPAttribute attrAd2 = new LDAPAttribute("address2", 
                                                    user.getAddress2());
        mods.add(LDAPModification.REPLACE, attrAd2);
        LDAPAttribute attrCity = new LDAPAttribute("city", user.getCity());
        mods.add(LDAPModification.REPLACE, attrCity);
        LDAPAttribute attrState = new LDAPAttribute("state", user.getState());
        mods.add(LDAPModification.REPLACE, attrState);
        LDAPAttribute attrZip = new LDAPAttribute("zip", user.getZip());
        mods.add(LDAPModification.REPLACE, attrZip);
        LDAPAttribute attrCC = new LDAPAttribute("creditCard", user.getCreditCard());
        mods.add(LDAPModification.REPLACE, attrCC);

        try {
            ldap.modify(userDN, mods);
        } catch (LDAPException e) {
            LogChannel chan = LBS.getLogChannel();
            if (chan != null)
                chan.write(Logger.ERROR, "LDAP error updating user: " + 
                           lookupLdapError(e) + 
                           "\n    User DN = " + userDN, e);
        }
        closeLdapConnection(ldap);
    }

    //----------------------------------------------------------

    private static LDAPConnection openLdapConnection() {
        LDAPConnection ldap = new LDAPConnection();
        try {
            ldap.connect(serverName, serverPort);
        } catch (LDAPException e) {
            LogChannel chan = LBS.getLogChannel();
            if (chan != null)
                chan.write(Logger.ERROR, "LDAP connect error: " + 
                           lookupLdapError(e), e);
            return null;
        }
        try {
            ldap.authenticate(adminUsername, adminPW);
        } catch (LDAPException e) {
            LogChannel chan = LBS.getLogChannel();
            if (chan != null)
                chan.write(Logger.ERROR, "LDAP authentication error: " + 
                           lookupLdapError(e), e);
            return null;
        }
        return ldap;
    }


    private static void closeLdapConnection (LDAPConnection ldap) {
        try {
            ldap.disconnect();
        } catch (LDAPException e) {
            LogChannel chan = LBS.getLogChannel();
            if (chan != null)
                chan.write(Logger.ERROR, "LDAP disconnect error: " + 
                           lookupLdapError(e), e);
        }
    }


    private static String lookupLdapError(LDAPException e) {
        String result = "";
        String s = e.getLDAPErrorMessage();
        if (s != null) 
            result += s + "\n";
        result += lookupLdapError(e.getLDAPResultCode());
        return result;
    }
 
        

    private static String lookupLdapError(int code) {
        switch (code) {
            case LDAPException.ALIAS_DEREFERENCING_PROBLEM: 
                return "An error occurred when dereferencing an alias.";
            case LDAPException.ALIAS_PROBLEM: 
                return "An problem occurred with an alias.";
            case LDAPException.ATTRIBUTE_OR_VALUE_EXISTS: 
                return "The value that you are adding to an attribute already exists in the attribute.";
            case LDAPException.AUTH_METHOD_NOT_SUPPORTED: 
                return "The specified authentication method is not supported by the LDAP server that you are connecting to.";
            case LDAPException.BUSY: 
                return "The DSA is busy.";
            case LDAPException.COMPARE_FALSE: 
                return "Value returned by an LDAP compare operation if the specified attribute and value is not found in the entry (no matching value found).";
            case LDAPException.COMPARE_TRUE: 
                return "Value returned by an LDAP compare operation if the specified attribute and value is found in the entry (matching value found).";
            case LDAPException.CONNECT_ERROR: 
                return "Your LDAP client failed to connect to the LDAP server.";
            case LDAPException.CONSTRAINT_VIOLATION: 
                return "An internal error occurred in the LDAP server.";
            case LDAPException.ENTRY_ALREADY_EXISTS: 
                return "The specified entry already exists.";
            case LDAPException.INAPPROPRIATE_AUTHENTICATION: 
                return "The authentication presented to the server is inappropriate.";
            case LDAPException.INAPPROPRIATE_MATCHING: 
                return "An inappropriate type of matching was used.";
            case LDAPException.INSUFFICIENT_ACCESS_RIGHTS: 
                return "The client is authenticated as a user who does not have the access privileges to perform this operation.";
            case LDAPException.INVALID_ATTRIBUTE_SYNTAX: 
                return "The request contains invalid syntax.";
            case LDAPException.INVALID_CREDENTIALS: 
                return "The credentials presented to the server for authentication are not valid.";
            case LDAPException.INVALID_DN_SYNTAX: 
                return "The specified distinguished name (DN) uses invalid syntax.";
            case LDAPException.IS_LEAF: 
                return "The specified entry is a leaf entry (it has no entries beneath it in the directory tree).";
            case LDAPException.LDAP_PARTIAL_RESULTS: 
                return "The LDAP server is referring your client to another LDAP server.";
            case LDAPException.LOOP_DETECT: 
                return "A loop has been detected.";
            case LDAPException.NAMING_VIOLATION: 
                return "A naming violation has occurred.";
            case LDAPException.NO_SUCH_ATTRIBUTE: 
                return "The specified attribute could not be found.";
            case LDAPException.NO_SUCH_OBJECT: 
                return "The entry specified in the request does not exist.";
            case LDAPException.NOT_ALLOWED_ON_NONLEAF: 
                return "The requested operation can only be performed on an entry that has no entries beneath it in the directory tree (in other words, a leaf entry).";
            case LDAPException.NOT_ALLOWED_ON_RDN: 
                return "The specified operation cannot be performed on a relative distinguished name (RDN).";
            case LDAPException.OBJECT_CLASS_MODS_PROHIBITED: 
                return "You cannot modify the specified object class.";
            case LDAPException.OBJECT_CLASS_VIOLATION: 
                return "The requested operation will add or change data so that the data no longer complies with the schema.";
            case LDAPException.OPERATION_ERROR: 
                return "An internal error occurred in the LDAP server.";
            case LDAPException.OTHER: 
                return "Other types of errors.";
            case LDAPException.PARAM_ERROR: 
                return "When calling a constructor or method from your client, one or more parameters were incorrectly specified.";
            case LDAPException.PROTOCOL_ERROR: 
                return "A LDAP server could not correctly interpret the request sent by your client because the request does not strictly comply with the LDAP protocol.";
            case LDAPException.SERVER_DOWN: 
                return "The LDAP server cannot be contacted.";
            case LDAPException.SIZE_LIMIT_EXCEEDED: 
                return "The search found more than the maximum number of results.";
            case LDAPException.STRONG_AUTH_REQUIRED: 
                return "A stronger authentication method (more than LDAP_AUTH_SIMPLE) is required by the LDAP server that you are connecting to.";
            case LDAPException.SUCCESS: 
                return "The operation completed successfully.";
            case LDAPException.TIME_LIMIT_EXCEEDED: 
                return "The search operation could not be completed within the maximum time limit.";
            case LDAPException.UNAVAILABLE: 
                return "The DSA is unavailable.";
            case LDAPException.UNDEFINED_ATTRIBUTE_TYPE: 
                return "The specified attribute is not defined.";
            case LDAPException.UNWILLING_TO_PERFORM: 
                return "The DSA is unable to perform the specified operation.";
            default:
                return "Unknown LDAP error.";
        }
    }
}


